/*
    RawSpeed - RAW file decoder.

    Copyright (C) 2016-2017 Roman Lebedev

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#pragma once

#if defined(__SSE2__)
#define WITH_SSE2
#else
/* #undef WITH_SSE2 */
#endif

#define HAVE_PUGIXML

#define HAVE_PTHREAD

#define HAVE_PUGIXML

#define HAVE_ZLIB

#define HAVE_JPEG
#define HAVE_JPEG_MEM_SRC

#define HAVE_CXX_THREAD_LOCAL
/* #undef HAVE_GCC_THREAD_LOCAL */

// which aligned memory allocation function is avaliable, if any?
// only the first one found will be enabled
#define HAVE_POSIX_MEMALIGN
/* #undef HAVE_ALIGNED_ALLOC */
/* #undef HAVE_MM_MALLOC */
/* #undef HAVE_ALIGNED_MALLOC */

/* #undef RAWSPEED_STANDALONE_BUILD */
#ifdef RAWSPEED_STANDALONE_BUILD
#define CMAKE_SOURCE_DIR "/builddir/build/BUILD/darktable-2.4.2"
#else
// If rawspeed is being built as part of some larger build, we can not retain
// the CMAKE_SOURCE_DIR, because that would affect the reproducible builds.
#endif

// see http://clang.llvm.org/docs/LanguageExtensions.html
#ifndef __has_feature      // Optional of course.
#define __has_feature(x) 0 // Compatibility with non-clang compilers.
#endif
#ifndef __has_extension
#define __has_extension __has_feature // Compatibility with pre-3.0 compilers.
#endif

// see https://github.com/google/sanitizers/wiki/AddressSanitizerManualPoisoning
#if __has_feature(address_sanitizer) || defined(__SANITIZE_ADDRESS__)
#include <sanitizer/asan_interface.h> // IWYU pragma: export
#include <type_traits>                // for remove_cv
template <typename T> struct remove_all_const : std::remove_const<T> {};
template <typename T> struct remove_all_const<T *> {
  using type = typename remove_all_const<T>::type *;
};
template <typename T> struct remove_all_const<T *const> {
  using type = typename remove_all_const<T>::type *;
};
#define ASAN_REGION_IS_POISONED(addr, size)                                    \
  __asan_region_is_poisoned(                                                   \
      const_cast<remove_all_const<decltype(addr)>::type>(addr),  /* NOLINT */  \
      (size))
#else
#define ASAN_POISON_MEMORY_REGION(addr, size) ((void)(addr), (void)(size))
#define ASAN_UNPOISON_MEMORY_REGION(addr, size) ((void)(addr), (void)(size))
#define ASAN_REGION_IS_POISONED(addr, size) (0)
#endif

#if __has_feature(memory_sanitizer) || defined(__SANITIZE_MEMORY__)
#include <sanitizer/msan_interface.h> // IWYU pragma: export
#define MSAN_MEM_IS_INITIALIZED(addr, size)                                    \
  __msan_check_mem_is_initialized(addr, size)
#else
#define MSAN_MEM_IS_INITIALIZED(addr, size) (0)
#endif
